% solve Schrodinger equation for one-dimensional square potential well 
% using finite differences
% Kyle Konrad 11/15/09
% Kyle 11/19/09
% Dhrubo 11/22/09
% Kyle 11/22/09


N = 10000; % degrees of freedom
xs = 0:1/N:1; %1-D mesh
k = 1000; % find the lowest k eigenvalues and eigenstates

% A is the finite difference approximation of the Laplacian
A = sparse(N,N);
A(1,1) = -2;
A(2,1) = 1;
A(N+1,N+1) = -2;
A(N+1,N) = 1;
for n = 2:N
    A(n,n) = -2;
    A(n-1,n) = 1;
    A(n+1,n) = 1;
end
A = N^2*A;

% compute eigenfunctions and eigenvalues
% A is sparse so we use eigs rather than eig
[v,e] = eigs(A,k,0);
% apparently eigs is nondeterministic. eigenvalues are different every time

% normalize
for n = 1:k
    v(:,n) = v(:,n).^2 / (sum(1/N*v(:,n).^2)); % v(:,n) is now psi_n^2
end
e = diag(e/e(1)); % e is now a list of normalized eigenvalues (e(1) = 1)


% Eigenvalues - error analysis
exp = (1:k)'.^2; % expected eigenvalues
eerrors = abs(exp - e)./exp; % eigenvalue errors

%Eigenfunctions
f = @(l, x) sin(l*pi*x); % helper function for generating eigenfunctions
psi = zeros(N+1,k);
dev = zeros(1,k);
for n = 1:k
    psi(:,n) = f(n,0:1/N:1); % exact eigenfunction for quantum number n
    psi(:,n) = (psi(:,n)/(sqrt((1/N)*sum(psi(:,n).^2)))).^2; % exact normalized probability distribution
    
    
    % Error analysis of eigenfunctions
    dev(n) = sqrt(sum((psi(:,n) - v(:,n)).^2)); % rms average of errors
end

% plot probability distributions of first 3 eigenfunctions
figure;
for n = 1:3
    plot(xs, v(:,n),'g'); hold on; % computed eigenfunctions
    plot(xs,psi(:,n),'r--'); hold on; % expected eigenfunctions
end
title('eigenfunctions');

figure; plot(e,'g');hold on;plot(exp,'r'); title('eigenvalues');
figure; plot(eerrors); title('eigenvalue errors');
figure; plot(dev); title('eigenfunction deviation');
