% find eigenfunctions and eigenvalues of stadium billiard
% Kyle Konrad 11/25/09
% update Dhrubo Jyoti 11/26/09
% update Kyle Konrad 11/27/09


% xlength = width of center rectancle of stadium
% M = degrees of freedom in y-direction
% k = number of eigenvalues to be found
% opt = 1: compute eigenvalues
%       0: don't compute eigenvalues


% xlength = 1;
% M = 100;
% k = 200;
% opt = 1;
% sigma = 0;

function [v,e] = stadeigs(xlength,M,k,sigma,opt)

if mod(M,2) ~= 0
    error('M must be even');
end

N = round((xlength+1)*M); % degrees of freedom in x-direction

if mod(N,2) ~= 0
    N = N+1; % make N even
end

hy = 1/(M+1); % vertical lattice spacing
hx = (xlength+1)/(N+1); % horizontal lattice spacing

isInSquare = @(x) -xlength/2 <= x(1) & x(1) <= xlength/2 & -0.5 <= x(2) & x(2) <= 0.5;
isInLeftCirc = @(x) (x(1)+xlength/2).^2 + x(2).^2<=.25;
isInRightCirc = @(x) (x(1)-xlength/2).^2 + x(2).^2<=.25;

isIn = @(x) isInSquare(x) | isInLeftCirc(x) | isInRightCirc(x);


y2row = @(y) round(-y/hy+M/2+1);
x2col = @(x) round(x/hx+N/2+1);

% populate xlist and ylist
xlist = zeros([N*M,1]); % larger than necessary
ylist = zeros([N*M,1]); % larger than necessary
indexlist = zeros(M+1,N+1);
cur=0;
[X,Y] = meshgrid((-N/2:N/2)*hx,(-M/2:M/2)*-hy);
for i = 1:(N+1)*(M+1)
    x=X(i);y=Y(i);
    if isIn([x,y])
        cur = cur+1;
        xlist(cur)=x;
        ylist(cur)=y;
        indexlist(y2row(y),x2col(x))=cur;
    end
end


grid2Index = @(x) indexlist(y2row(x(2)),x2col(x(1)));

% A is the finite difference approximation of the Laplacian in 2-D
% where columns in region are stached into a single column of A

%   ^                  
% y |                  
%   |      1           
%   |  1  -4   1       
%   |      1                
%   |                  
%    ------------>
%              x

s = cur; % number of lattice points in stadium
A = sparse(s,s);

for n = 1:s
        xn = xlist(n);
        yn = ylist(n);
        
        west = [xn-hx,yn];
        east = [xn+hx,yn];
        north = [xn,yn+hy];
        south = [xn,yn-hy];
        
        
        A(n,n) = -4; %self
        if isIn(west); A(n,grid2Index(west))= 1; end; %west
        if isIn(east); A(n,grid2Index(east))= 1; end; %east
        if isIn(north); A(n,grid2Index(north))= 1; end; %north
        if isIn(south); A(n,grid2Index(south))= 1; end; %south
end

% compute eigenfunctions and eigenvalues
opts.disp = 0;
if opt == 0
     tic;
     e = sort(eigs(-A,k,sigma,opts));
     fprintf(1,'eigs: %d\n', toc);
     v = [];
else tic;
     [v,e] = eigs(-A,k,sigma,opts);
     fprintf(1,'eigs: %d\n', toc);
     v(s+1,:) = diag(e);
     t = sortrows(v',s+1)';
     e = t(s+1,:)'; % column vector of sorted eigenvalues
     v = t(1:s,:); % corresponding eigenvectors


     % reshape eigenvectors
     tic;
     S = zeros(M+2,N+2,k);
     [c,r] = meshgrid(1:k,1:s);
     for i = 1:k*s
         S(y2row(ylist(r(i))),x2col(xlist(r(i))), c(i)) = v(r(i),c(i));
     end
     fprintf(1,'reshape: %d\n', toc);
     v = S;
     
end